==> lowcode-render-1.1.2.tgz: package/src/utils/debugExpr.ts <==
import { PropBindType } from '@cloudbase/cals';
import { observable, reaction, remove, untracked } from 'mobx';
import { EventProxy } from './eventProxy';
import { generateDataBinds } from './adapter/generate';

/**
 * 以 widget id 为 key，只记录同个 widget 的最后一次表达式的结果
 * 这里因为同时只会执行（调试）一个表达式，所以只需保存最后一次执行的结果即可
 */

const eventProxy = new EventProxy();

export let debugExprMap = observable<{
  node: {
    [nodeId: string]: {
      expr: string;
      result: any;
      refreshSigh: number;
      lastForDebugExprDispose: () => any;
    };
  };
}>({ node: {} });

export function resetDebugExprMap() {
  // FIXME: widgets 中 for 之下 widget 的 dispose 没有被触发，所以这边通过 remove 来删除使原来的 autorun 不再触发
  remove(debugExprMap, 'node');
  debugExprMap.node = {};
  window['__debugExprMap'] = debugExprMap;
}

export function watchForDebugExpr({
  widgetId: paramWidgetId,
  subForItems,
  globalCtx,
  getDataContextFn,
  get$wFn,
  get$scopeFn,
}) {
  // FIXME: 暂时没考虑低码组件开发模式
  const dataContext = untracked(getDataContextFn);
  const $w = untracked(get$wFn);
  const $scope = untracked(get$scopeFn);

  // 以下代码为最终的版本，可支持任一级
  // const widgetId = paramWidgetId + (subForItems?.forIndexes?.length ? `_${subForItems?.forIndexes.join('_')}` : '_0');
  // 以下代码为只支持第一级循环的版本（因为当前编辑器只支持选择第一级循环项的 2, ...n 项，不支持选择第二层（及以下）循环项的 2, ...n 项）
  const widgetId = paramWidgetId + `_${subForItems?.forIndexes?.[0] || 0}`;

  if (!debugExprMap.node[widgetId]) {
    debugExprMap.node[widgetId] = {
      expr: '',
      result: '',
      refreshSigh: +new Date(),
      lastForDebugExprDispose: null,
    };
  }

  if (debugExprMap.node[widgetId].lastForDebugExprDispose) {
    debugExprMap.node[widgetId].lastForDebugExprDispose();
  }
  const forDebugExprDispose = reaction(
    // 监听 refreshSigh 的变化
    () => [debugExprMap.node[widgetId]?.refreshSigh, debugExprMap.node[widgetId]?.expr],
    ([_, expr]: [number, string]) => {
      if (expr?.trim()) {
        const fn = generateDataBinds(globalCtx, [
          {
            type: PropBindType.expression,
            bindDataPath: expr,
            propertyPath: 'expr',
          },
        ])?.expr;
        if (fn) {
          try {
            const result = fn(subForItems, undefined, dataContext, $w, $scope);
            debugExprMap.node[widgetId].result = result;
          } catch (e) {
            debugExprMap.node[widgetId].result = e;
          }
        } else {
          debugExprMap.node[widgetId].result = '';
        }
      } else {
        if (debugExprMap.node?.[widgetId]) debugExprMap.node[widgetId].result = '';
      }

      eventProxy.emit(`done_${widgetId}_${expr}`);
    },
  );

  debugExprMap.node[widgetId].lastForDebugExprDispose = forDebugExprDispose;
  return forDebugExprDispose;
}

/**
 * @param widgetId: 当在循环中，需要将所在索引信息进行拼接。如 text1_0（借鉴 0 的设计，非循环项的 widgetId 这里需自动加 0，如 button1_0）
 */
export function evalExpr(widgetId: string, expr: string, useCache = false) {
  if (!/.*_\d+$/.test(widgetId)) {
    // 兼容旧的版本
    widgetId = widgetId + '_0';
  }

  return new Promise((resolve) => {
    if (!debugExprMap.node[widgetId]) {
      resolve(undefined);
      return;
    }

    if (useCache && debugExprMap.node[widgetId].expr === expr) {
      // 表达式没变，直接返回结果
      resolve(debugExprMap.node[widgetId].result);
    } else {
      eventProxy.once(`done_${widgetId}_${expr}`, () => {
        resolve(debugExprMap.node[widgetId].result);
      });

      // 触发 autorun
      if (!useCache && debugExprMap.node[widgetId].expr === expr) {
        debugExprMap.node[widgetId].refreshSigh = +new Date();
      } else {
        debugExprMap.node[widgetId].expr = expr || '';
      }
    }
  });
}

window['__testExpr'] = evalExpr;

==> lowcode-render-1.1.3.tgz: package/src/utils/debugExpr.ts <==
import { PropBindType } from '@cloudbase/cals';
import { observable, reaction, remove, untracked } from 'mobx';
import { EventProxy } from './eventProxy';
import { generateDataBinds } from './adapter/generate';
import { runtime } from './adapter';

/**
 * 以 widget id 为 key，只记录同个 widget 的最后一次表达式的结果
 * 这里因为同时只会执行（调试）一个表达式，所以只需保存最后一次执行的结果即可
 */

const eventProxy = new EventProxy();

export let debugExprMap = observable<{
  node: {
    [nodeId: string]: {
      expr: string;
      result: any;
      refreshSigh: number;
      lastForDebugExprDispose: () => any;
    };
  };
}>({ node: {} });

export function resetDebugExprMap() {
  // FIXME: widgets 中 for 之下 widget 的 dispose 没有被触发，所以这边通过 remove 来删除使原来的 autorun 不再触发
  remove(debugExprMap, 'node');
  debugExprMap.node = {};
  runtime.__debugExprMap = debugExprMap;
}

export function watchForDebugExpr({
  widgetId: paramWidgetId,
  subForItems,
  globalCtx,
  getDataContextFn,
  get$wFn,
  get$scopeFn,
}) {
  // FIXME: 暂时没考虑低码组件开发模式
  const dataContext = untracked(getDataContextFn);
  const $w = untracked(get$wFn);
  const $scope = untracked(get$scopeFn);

  // 以下代码为最终的版本，可支持任一级
  // const widgetId = paramWidgetId + (subForItems?.forIndexes?.length ? `_${subForItems?.forIndexes.join('_')}` : '_0');
  // 以下代码为只支持第一级循环的版本（因为当前编辑器只支持选择第一级循环项的 2, ...n 项，不支持选择第二层（及以下）循环项的 2, ...n 项）
  const widgetId = `${paramWidgetId}_${subForItems?.forIndexes?.[0] || 0}`;

  if (!debugExprMap.node[widgetId]) {
    debugExprMap.node[widgetId] = {
      expr: '',
      result: '',
      refreshSigh: +new Date(),
      lastForDebugExprDispose: null,
    };
  }

  if (debugExprMap.node[widgetId].lastForDebugExprDispose) {
    debugExprMap.node[widgetId].lastForDebugExprDispose();
  }
  const forDebugExprDispose = reaction(
    // 监听 refreshSigh 的变化
    () => [debugExprMap.node[widgetId]?.refreshSigh, debugExprMap.node[widgetId]?.expr],
    ([_, expr]: [number, string]) => {
      if (expr?.trim()) {
        const fn = generateDataBinds(globalCtx, [
          {
            type: PropBindType.expression,
            bindDataPath: expr,
            propertyPath: 'expr',
          },
        ])?.expr;
        if (fn) {
          try {
            const result = fn(subForItems, undefined, dataContext, $w, $scope);
            debugExprMap.node[widgetId].result = result;
          } catch (e) {
            debugExprMap.node[widgetId].result = e;
          }
        } else {
          debugExprMap.node[widgetId].result = '';
        }
      } else {
        if (debugExprMap.node?.[widgetId]) debugExprMap.node[widgetId].result = '';
      }

      eventProxy.emit(`done_${widgetId}_${expr}`);
    },
  );

  debugExprMap.node[widgetId].lastForDebugExprDispose = forDebugExprDispose;
  return forDebugExprDispose;
}

/**
 * @param widgetId: 当在循环中，需要将所在索引信息进行拼接。如 text1_0（借鉴 0 的设计，非循环项的 widgetId 这里需自动加 0，如 button1_0）
 */
export function evalExpr(widgetId: string, expr: string, useCache = false) {
  if (!/.*_\d+$/.test(widgetId)) {
    // 兼容旧的版本
    widgetId = `${widgetId}_0`;
  }

  return new Promise((resolve) => {
    if (!debugExprMap.node[widgetId]) {
      resolve(undefined);
      return;
    }

    if (useCache && debugExprMap.node[widgetId].expr === expr) {
      // 表达式没变，直接返回结果
      resolve(debugExprMap.node[widgetId].result);
    } else {
      eventProxy.once(`done_${widgetId}_${expr}`, () => {
        resolve(debugExprMap.node[widgetId].result);
      });

      // 触发 autorun
      if (!useCache && debugExprMap.node[widgetId].expr === expr) {
        debugExprMap.node[widgetId].refreshSigh = +new Date();
      } else {
        debugExprMap.node[widgetId].expr = expr || '';
      }
    }
  });
}

(window as any).__testExpr = evalExpr;
